// #include & #define
#include <Wire.h>
#include <VL53L0X.h>
#include <Adafruit_NeoPixel.h>
#include <SoftwareSerial.h>
#include <DFRobotDFPlayerMini.h>

//#define HIGH_SPEED // Interchangeable with '#define HIGH_ACCURACY'
#define HIGH_ACCURACY
#define LONG_RANGE
#define PIN    3
#define N_LEDS 57

VL53L0X sensor;
Adafruit_NeoPixel strip = Adafruit_NeoPixel(N_LEDS, PIN, NEO_GRB + NEO_KHZ800);
SoftwareSerial mySerial(5, 6); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

float distance = 0;
int color = strip.Color(225, 205, 255);
int null = strip.Color(0, 0, 0);
int ringSizes[] = {20, 16, 12, 8, 1}; // Number of pixels in each ring
float delayTime = 0;

// setup() Arduino Uno R3 & 'Serial.begin();'
void setup() {
  Serial.begin(9600);
  setup_ToF();
  setup_LED();
  setup_MP3();
}

void setup_ToF() {
    Wire.begin();

  sensor.setTimeout(500);
  if (!sensor.init()) {
    Serial.println("Failed to detect and initialize sensor!");
    while (1) {}
  }

  #if defined LONG_RANGE
    // lower the return signal rate limit (default is 0.25 MCPS)
    sensor.setSignalRateLimit(0.1);
    // increase laser pulse periods (defaults are 14 and 10 PCLKs)
    sensor.setVcselPulsePeriod(VL53L0X::VcselPeriodPreRange, 18);
    sensor.setVcselPulsePeriod(VL53L0X::VcselPeriodFinalRange, 14);
  #endif

  #if defined HIGH_SPEED
    // reduce timing budget to 20 ms (default is about 33 ms)
    sensor.setMeasurementTimingBudget(20000);
  #elif defined HIGH_ACCURACY
    // increase timing budget to 100 ms
    sensor.setMeasurementTimingBudget(100000);
  #endif
}

void setup_LED() {
  strip.begin();
  strip.setBrightness(10);
}

void setup_MP3() {
  mySerial.begin(9600);
  Serial.begin(115200);
  Serial.println();
  Serial.println("Initializing DFPlayer ...");
  if (!myDFPlayer.begin(mySerial)) {  // Use softwareSerial to communicate with mp3.
    Serial.println("Unable to begin DFPlayer.");
    while (true){
      delay(0); // Code to compatible with ESP8266 watch dog.
    }
  }

  Serial.println("DFPlayer Mini online.");
  myDFPlayer.volume(100);  // Set volume to 100%
  myDFPlayer.play(4);     // Play "startup.wav"
  delay(3000);
}

void loop() {  // run loops
  loop_ToF();
  loop_MP3();

  if(distance<=1500) {
    delayTime = 180;
  }
  if(distance<=1000) {
    delayTime = 90;
  }
  if(distance<=500) {
    delayTime = 30;
  }

  // loop LED
  if (distance <= 1500) {
    for (int ring=4; ring>=0; ring--) {
      // Calculate start and end pixel indices for current ring
      int startPixel = 0;
      for (int i=0; i<ring; i++) {
        startPixel += ringSizes[i];
      }

      // Fade out previous rings
      int endPixel = startPixel + ringSizes[ring] - 1;
      for (int j = ring + 1; j <= min(4, ring + 3); j++) {
        int prevStartPixel = 0;
        for (int i = 0; i < j; i++) {
          prevStartPixel += ringSizes[i];
        }

        int prevEndPixel = prevStartPixel + ringSizes[j] - 1;
        for (int i = prevStartPixel; i <= prevEndPixel; i++) {
          int prevPixelColor = strip.getPixelColor(i);
          int brightness = (255 - (j - ring) * 75) / 255;
          strip.setPixelColor(i, brightness, brightness, brightness);
        }
      }

      // Set current ring to a color
      for (int i = startPixel; i < startPixel + ringSizes[ring]; i++) { // increment by the size of the current ring
        strip.setPixelColor(i, color);
      }

      strip.show();
      delay(delayTime);

      // Turn off current ring
      for (int i = startPixel; i <= endPixel; i++) {
        strip.setPixelColor(i, null);
      }

      // Fade out fill-in for trailing ring
      if (ring == 0) {
        for (int b = 255; b >= 0; b--) {
          for (int i = startPixel; i <= endPixel; i++) {
            strip.setPixelColor(i, b, b, b);
            strip.setPixelColor(56, b, b, b);
          }
          strip.show();
        }
      }

      // Turn off previous rings
      for (int j = ring + 1; j <= min(4, ring + 3); j++) {
        int prevStartPixel = 0;
        for (int i = 0; i < j; i++) {
          prevStartPixel += ringSizes[i];
        }
        int prevEndPixel = prevStartPixel + ringSizes[j] - 1;
        for (int i = prevStartPixel; i <= prevEndPixel; i++) {
          strip.setPixelColor(i, null);
        }
      }
    }

    strip.show();
    delay(delayTime);
    Serial.print("Threat detected ");
    Serial.print(distance);
    Serial.println("mm");
  }
}

void loop_ToF() {
  distance = sensor.readRangeSingleMillimeters();

  if (sensor.timeoutOccurred()) {
    Serial.print(" TIMEOUT");
  }

  Serial.println();
}

// MP3 Code
void loop_MP3() {
  if (distance<=1500 && distance>1000) {
    myDFPlayer.play(3);     // Play "blipNorm.wav"
    // delay(2500);
  }
  if(distance<=1000 && distance>500) {
    myDFPlayer.play(1);     // Play "blipNear.wav"
    // delay(1400);
  }
  if (distance<=500) {
    myDFPlayer.play(2);     // Play "blipClose.wav"
    // delay(1020);
  }
}